////////////////////////////////////////////////////////////////////////////
////                             LCD.C                                  ////
////                 Driver for common LCD modules                      ////
////                                                                    ////
////  lcd_init()   Must be called before any other function.            ////
////                                                                    ////
////  lcd_putc(c)  Will display c on the next position of the LCD.      ////
////                     The following have special meaning:            ////
////                      \f  Clear display                             ////
////                      \n  Go to start of second line                ////
////                      \b  Move back one position                    ////
////                                                                    ////
////  lcd_gotoxy(x,y) Set write position on LCD (upper left is 1,1)     ////
////                                                                    ////
////  lcd_getc(x,y)   Returns character at position x,y on LCD          ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////
// Krzysztof Pawsiuk (2002)- Procedura obslugiwyswietlacza LCD zostala ////
// stworzona na bazie procedury udostepnianej wraz z kompilatorem.		////
// Procedura zostala w znacznym stopniu przebudowana, najwazniejsza 	////
// zmiana umozliwia dowolne przypisanie pinow sterujacych wyswietlaczem ////
// poszczegolne linie danych lub sterujace moga pochodzic z roznych     ////
// portow.																////
////////////////////////////////////////////////////////////////////////////

//*************************************************************************//
//	Deklaracja parametrow startowych niezbednych do sterowania			   //
//  wyswietlaczem									   					   //
//*************************************************************************//
#ifndef lcd
   #define 	lcd_en  			porta_2
   #define	lcd_tris_en 		trisa_2
   #define 	lcd_rs				porta_0
   #define	lcd_tris_rs			trisa_0
   //#define 	lcd_rw				porta_1
   //#define	lcd_tris_rw			trisa_1
   #define   lcd_d7        		portb_7
   #define   lcd_tris_d7   		trisb_7
   #define   lcd_d6        		portb_6
   #define   lcd_tris_d6   		trisb_6
   #define   lcd_d5        		portb_5
   #define   lcd_tris_d5   		trisb_5
   #define   lcd_d4        		portb_4
   #define   lcd_tris_d4   		trisb_4
#endif

#define lcd_type 2           // 0=5x7, 1=5x10, 2=2 lines
#define lcd_line_two 0x40    // LCD RAM address for the second line

//male litery, wszystkie polskie znaki maja wartosci wieksze od 139
//dlatego do wykrycia czy to jest polski znak sprawdzane jest czy jest
//od 139
#define _a			0xB9			//
#define _e			0xEA			//
#define _n			0xF1			//
#define _z			0xBF			//
#define _c			0xE6			//
#define _s			0x9C			//
//#define _x			0x9F			//
#define _o			0xF3			//
#define _l			0xB3			//


/////////////////////////////////////////////////////////////////////////
byte const LcdDefLow[64]={0x00,0x00,0x0E,0x01,0x0F,0x11,0x0F,0x02,	//
					   0x00,0x00,0x0E,0x11,0x1F,0x10,0x0E,0x02,     //
					   0x01,0x02,0x16,0x19,0x11,0x11,0x11,0x00,     //
					   0x00,0x04,0x1F,0x02,0x04,0x08,0x1F,0x00,     //
					   0x02,0x04,0x0E,0x10,0x10,0x11,0x0E,0x00,     //
					   0x02,0x04,0x0E,0x10,0x0E,0x01,0x1E,0x00,	    //
					   0x02,0x04,0x0E,0x11,0x11,0x11,0x0E,0x00,	    //
					   0x0C,0x04,0x06,0x0C,0x04,0x04,0x0E,0x00};	//
////////////////////////////////////////////////////////////////////////////

byte const DefLow[8]={_a,_e,_n,_z,_c,_s,_o,_l};


byte	lcd_nr,licz; 			//okresla jaki jest aktualny adres znaku w LCD


#ifdef lcd_rw
byte lcd_read_byte() {
      byte low,high;

      lcd_tris_d7=1;lcd_tris_d6=1;lcd_tris_d5=1;lcd_tris_d4=1; //piny danych jako wejscia
      lcd_rw = 1;
      delay_cycles(1);
      lcd_en = 1;
      delay_cycles(1);
      high=0;              			//odzczytuje stany wszystkich wejsc danych
      if(lcd_d4) high+=1;           //wynik jest w high, aby odczytac 4 bity trzeba dodawac
      if(lcd_d5)           			//poszczegolne bity wedlog ich wagi, dlatego, ze kazdy bit
         high+=2;          			//moze byc z innego portu
      if(lcd_d6) high+=4;
      if(lcd_d7) high+=8;
      lcd_en = 0;
      delay_cycles(1);
      lcd_en = 1;
      delay_us(1);
      low=0;
      if(lcd_d4) low+=1;   			//odczytaj druga tetrade i zpisz wynik w low
      if(lcd_d5) low+=2;
      if(lcd_d6) low+=4;
      if(lcd_d7) low+=8;
      lcd_en = 0;
      //set_tris_b(LCD_WRITE);
      lcd_tris_d7=0;lcd_tris_d6=0;lcd_tris_d5=0;lcd_tris_d4=0; //piny danych jako wyjscia
      return( (high<<4) | low);
}
#endif
//  wysyla mlodsza tetrade bajtu n do wyswietlacza
void lcd_send_nibble( byte nn ) {
      lcd_tris_d4=0;
      lcd_d4=bit_test(nn,0);
      lcd_d5=bit_test(nn,1);
      lcd_d6=bit_test(nn,2);
      lcd_d7=bit_test(nn,3);
      delay_cycles(1);
      lcd_en = 1;
      delay_us(2);
      lcd_en = 0;
}
////////////////////////////////////////////////////////////////////////////////////////

void lcd_send_byte( byte address, byte nn ) {
	  lcd_tris_rs=0;
      lcd_rs = 0;
#ifdef lcd_rw
      while ( bit_test(lcd_read_byte(),7) ) ;//czekaj az wyswietlacz bedzie gotowy
 #else
      delay_us(40);     //jesli busy nie jest odczytywany z lcd to czekaj na wykonanie instrukcji
#endif                  //okolo 40us

      lcd_rs = address;
      delay_cycles(1);
#ifdef lcd_rw                 //jesli jest urzywany pin RW
      lcd_rw = 0;
#endif
      delay_cycles(1);
      lcd_en = 0;
      lcd_send_nibble(nn >> 4);
      lcd_send_nibble(nn & 0x0f);
}
///////////////////////////////////////////////////////////////////////////////////////////////
//inicjalizacja wyswietlacza - dziala
//  lcd_init();
/////////////////////////////////////////////////////////////////////////////////////////////
void lcd_init() {
    byte i;

    //set_tris_b(LCD_WRITE);
    lcd_d7=0;lcd_d6=0;lcd_d5=0;lcd_d4=0;                     //piny danych jako wyjscia
    lcd_tris_d7=0;lcd_tris_d6=0;lcd_tris_d5=0;lcd_tris_d4=0; //piny danych jako wyjscia
    lcd_rs = 0;
    lcd_tris_rs=0;                                           //piny seterujace=0
    lcd_en = 0;
    lcd_tris_en=0;
//jesli jest wykozystywany pin RW to zrob to
#ifdef lcd_rw
    lcd_rw = 0;
    lcd_tris_rw=0;                                          //piny sterujace jako wyjscia
#endif
//-----------------------------------------------------------------
    delay_ms(15);                                           //czekaj na start lcd
    for(i=1;i<=3;++i) {
       lcd_send_nibble(3);
       delay_ms(5);
    }
    lcd_send_nibble(2);
   lcd_send_byte(0,0x20 | (lcd_type << 2));
   lcd_send_byte(0,0xc);
   lcd_send_byte(0,1);
   delay_ms(2);
   lcd_send_byte(0,6);
   lcd_nr=0;

}
//////////////////////////////////////////////////////////////////////////////////////////////

// ustawia kursor na pozycji xy
void lcd_gotoxy( byte y, byte x) {
   byte address;
   if(y!=1)
     address=lcd_line_two;
   else
     address=0;
   address+=x-1;
   lcd_nr=address;											//zapisz adres kursora do lcd_nr
   lcd_send_byte(0,0x80|address);
}



//    wyswietla znaki na wyswietlaczu
void lcd_putc(int f)
{
int ia,ib;
     //switch(f)
     //{
     if(f=='\r')  {lcd_send_byte(0,1);			//clear LCD
                   lcd_nr=0;					//zapisuje do licznika miejsce kursora
                   delay_ms(2);return;
                  }
//prawdza czy nie mala polska litera, jesli tak to wyswietla ja
	 for(ia=0;ia<=7;ia++)
	 	if(f==defLow[ia])
	 		{lcd_send_byte(1,ia);
	 		return;}
	 lcd_send_byte(1,f);

}

//************************************************************************//
//    laduje do pamieci polskie znaki oprocz                             //
//************************************************************************//
void lcd_znaki()
{
int licz;
lcd_send_byte(0,0x40);  			  //ustawienie adresu pamieci lcd na zero
for(licz=0;licz<64;licz++)			  //wpisuje dane z tablicy do LCD
   lcd_send_byte(1,lcdDefLow[licz]);
 }

//************************************************************************//





